/*
	(C) 1995-2000 AROS - The Amiga Research OS
	$Id: screendepth.c,v 1.3 2008/08/26 21:03:46 jacadcaps Exp $

	Desc: Change order of screens.
	Lang: English
*/
#include "intuition_intern.h"
#include "inputhandler_actions.h"
#include "mosmisc.h"
#include "screendepth.h"

/*****************************************************************************

	NAME */
#include <proto/intuition.h>

AROS_LH3(void, ScreenDepth,

		 /*  SYNOPSIS */
		 AROS_LHA(struct Screen *, screen, A0),
		 AROS_LHA(ULONG          , flags, D0),
		 AROS_LHA(APTR           , reserved, A1),

		 /*  LOCATION */
		 struct IntuitionBase *, IntuitionBase, 131, Intuition)

/*  FUNCTION
	Move the specified screen to the front or back, based on passed flag.
	If the screen is in a group, the screen will change its position in
	the group only. If the screen is the parent of a group, the whole
	group will be moved.

	INPUTS
	screen - Move this screen.
	flags - SDEPTH_TOFRONT or SDEPTH_TOBACK for bringing the screen to
		front or back.
		If the screen is a child of another screen you may specify
		SDEPTH_INFAMILY to move the screen within the family. If
		not specified the whole family will move.
	reserved - For future use. MUST be NULL by now.

	RESULT
	None.

	NOTES
	Only the owner of the screen should use SDEPTH_INFAMILY.
	Intentionally commodities should not change the internal arrangement
	of screen families.

	EXAMPLE

	BUGS

	SEE ALSO
	ScreenToBack(), ScreenToFront()

	INTERNALS

	HISTORY

*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct IntuitionBase *,IntuitionBase)

	struct ScreenDepthActionMsg msg;

	if (reserved != NULL) return;
	SANITY_CHECK(screen)

	msg.screen = screen;
	msg.flags = flags;

	DoASyncAction((APTR)int_screendepthmos, &msg.msg, sizeof(msg), IntuitionBase);

	AROS_LIBFUNC_EXIT

} /* ScreenDepth */

/*****************************************************************************************/
